/********************************************************************************
 * Copyright (c) Erik Kunze 1996, 1997
 *
 * Permission to use, distribute, and sell this software and its documentation
 * for any purpose is hereby granted without fee, provided that the above
 * copyright notice appear in all copies and that both that copyright notice and
 * this permission notice appear in supporting documentation, and that the name
 * of the copyright holder not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission.  The
 * copyright holder makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 *
 * changed by EKU
 *******************************************************************************/
#ifdef AUDIO
#ifndef lint
static char rcsid[] = "$Id: audio.c,v 3.5 1997/03/16 08:18:42 erik Rel $";
#endif

#if defined(XBELL_AUDIO) && (defined(SUN_AUDIO) || defined(PCSPKR_AUDIO))
#ifdef __GNUC__
#error Speaker and X bell together not supported!
#else
#pragma error Speaker and X bell together not supported!
#endif
#endif
#if defined(PCSPKR_AUDIO) && !defined(__linux__)
#ifdef __GNUC__
#error Do not define PCSPKR_AUDIO on non Linux systems!
#else
#pragma error Do not define PCSPKR_AUDIO on non Linux systems!
#endif
#endif
#if defined(NAS_AUDIO) && (defined(AYCHIP_AUDIO) || defined(SUN_AUDIO) || \
						   defined(PCSPKR_AUDIO) || defined(XBELL_AUDIO))
#ifdef __GNUC__
#error Do not define NAS_AUDIO together with another XXX_AUDIO
#else
#pragma error Do not define NAS_AUDIO together with another XXX_AUDIO
#endif
#endif
#include "resource.h"
#include "util.h"
#include "audio.h"




#if defined(SUN_AUDIO) || (defined(AYCHIP_AUDIO) && defined(sun))

static unsigned char ulaw[256] = {
	 31,  31,  31,  32,  32,  32,  32,  33,
	 33,  33,  33,  34,  34,  34,  34,  35,
	 35,  35,  35,  36,  36,  36,  36,  37,
	 37,  37,  37,  38,  38,  38,  38,  39,
	 39,  39,  39,  40,  40,  40,  40,  41,
	 41,  41,  41,  42,  42,  42,  42,  43,
	 43,  43,  43,  44,  44,  44,  44,  45,
	 45,  45,  45,  46,  46,  46,  46,  47,
	 47,  47,  47,  48,  48,  49,  49,  50,
	 50,  51,  51,  52,  52,  53,  53,  54,
	 54,  55,  55,  56,  56,  57,  57,  58,
	 58,  59,  59,  60,  60,  61,  61,  62,
	 62,  63,  63,  64,  65,  66,  67,  68,
	 69,  70,  71,  72,  73,  74,  75,  76,
	 77,  78,  79,  81,  83,  85,  87,  89,
	 91,  93,  95,  99, 103, 107, 111, 119,
	255, 247, 239, 235, 231, 227, 223, 221,
	219, 217, 215, 213, 211, 209, 207, 206,
	205, 204, 203, 202, 201, 200, 199, 198,
	197, 196, 195, 194, 193, 192, 191, 191,
	190, 190, 189, 189, 188, 188, 187, 187,
	186, 186, 185, 185, 184, 184, 183, 183,
	182, 182, 181, 181, 180, 180, 179, 179,
	178, 178, 177, 177, 176, 176, 175, 175,
	175, 175, 174, 174, 174, 174, 173, 173,
	173, 173, 172, 172, 172, 172, 171, 171,
	171, 171, 170, 170, 170, 170, 169, 169,
	169, 169, 168, 168, 168, 168, 167, 167,
	167, 167, 166, 166, 166, 166, 165, 165,
	165, 165, 164, 164, 164, 164, 163, 163,
	163, 163, 162, 162, 162, 162, 161, 161,
	161, 161, 160, 160, 160, 160, 159, 159,
};
#endif

#ifdef AYCHIP_AUDIO
#include "audio/ay8912.c"
#endif
#ifdef SUN_AUDIO
#include "audio/sunaudio.c"
#endif
#ifdef PCSPKR_AUDIO
#include "audio/pcspkr.c"
#endif
#ifdef XBELL_AUDIO
#include "audio/beep.c"
#endif
#ifdef NAS_AUDIO
#include "audio/nas.c"
#endif

int
AudioInit(void)
{
	int ret = -1;
#ifdef AYCHIP_AUDIO
#ifdef SUN_AUDIO

	if (AyInit(GETCFG(audioDev), 0) != -1)
#else
	if (AyInit(GETCFG(audioDev), GETCFG(useSound)) != -1)
#endif
	{
		ret = 0;
	}
#endif
#ifdef PCSPKR_AUDIO
	if (SpeakerInit(GETCFG(useSound)) != -1)
	{
		ret = 0;
	}
#endif
#ifdef SUN_AUDIO

	if (SpeakerInit(GETCFG(audioDev), 0) != -1)
	{
		ret = 0;
	}
#endif
#ifdef XBELL_AUDIO
	ret = 0;
#endif
#ifdef NAS_AUDIO
	if (NasInit(GETCFG(useSound)) != -1)
	{
		ret = 0;
	}
#endif
	if (ret == -1)
	{
		Msg(M_WARN, "audio initialisation failed");
		SETCFG(useSound, False);
	}
#ifdef sun
	else
	{
		if (GETCFG(useSound))
		{
#if defined(AYCHIP_AUDIO) && defined(SUN_AUDIO)

			if (GETCFG(machine) == 48)
			{
				SpeakerOnOff();
			}
			else
			{
				AyOnOff();
			}
#elif defined(AYCHIP)
			AyOnOff();
#else
			SpeakerOnOff();
#endif
		}
	}
#endif
	return ret;
}
#endif

